@@ -42,7 +42,8 @@
 #include "autosave.h"
 #include "dynamic_refresh.h"
 #include "textedit.h"
-#include "diamarshal.h"
+#include "lib/diamarshal.h"
+#include "parent.h"
 
 static GList *open_diagrams = NULL;
 
@@ -59,6 +60,7 @@ static gint diagram_parent_sort_cb(gconstpointer a, gconstpointer b);
 
 static void diagram_class_init (DiagramClass *klass);
 static gboolean diagram_init(Diagram *obj, const char *filename);
+static void diagram_update_for_filename(Diagram *dia);
 
 enum {
   SELECTION_CHANGED,
@@ -251,16 +253,8 @@ diagram_load_into(Diagram         *diagram,
   if (!ifilter)  /* default to native format */
     ifilter = &dia_import_filter;
 
-  if (!diagram_init(diagram, filename)) {
-    return FALSE;
-  }
-
   if (ifilter->import_func(filename, diagram->data, ifilter->user_data)) {
-    diagram->unsaved = FALSE;
-    diagram_set_modified(diagram, FALSE);
-    if (app_is_interactive())
-      recent_file_history_add(filename);
-    diagram_tree_add(diagram_tree(), diagram);
+    diagram_set_modified(diagram, TRUE);
     return TRUE;
   } else
     return FALSE;
@@ -269,17 +263,42 @@ diagram_load_into(Diagram         *diagram,
 Diagram *
 diagram_load(const char *filename, DiaImportFilter *ifilter)
 {
-  Diagram *diagram;
+  Diagram *diagram = NULL;
+  GList *diagrams;
+  gboolean was_default = FALSE;
 
-  /* TODO: Make diagram not be initialized twice */
-  diagram = new_diagram(filename);
-  if (diagram == NULL) return NULL;
-
-  if (!diagram_load_into (diagram, filename, ifilter)) {
-    diagram_destroy(diagram);
-    diagram = NULL;
+  for (diagrams = open_diagrams; diagrams != NULL; diagrams = g_list_next(diagrams)) {
+    Diagram *old_diagram = (Diagram*)diagrams->data;
+    if (old_diagram->is_default) {
+      diagram = old_diagram;
+      was_default = TRUE;
+      break;
+    }	
   }
 
+  /* TODO: Make diagram not be initialized twice */
+  if (diagram == NULL) {
+    diagram = new_diagram(filename);
+  }
+  if (diagram == NULL) return NULL;
+
+  if (   !diagram_init(diagram, filename)
+      || !diagram_load_into (diagram, filename, ifilter)) {
+    diagram_destroy(diagram);
+    diagram = NULL;
+  } else {
+    diagram->unsaved = FALSE;
+    diagram_set_modified(diagram, FALSE);
+    if (app_is_interactive())
+      recent_file_history_add(filename);
+    diagram_tree_add(diagram_tree(), diagram);
+  }
+  
+  if (diagram != NULL && was_default) {
+    diagram_update_for_filename(diagram);
+    diagram->is_default = FALSE;
+  }
+  
   return diagram;
 }
 
@@ -328,7 +347,10 @@ diagram_modified(Diagram *dia)
     ddisplay_update_statusbar(display);
     displays = g_slist_next(displays);
   }
-  if (diagram_is_modified(dia)) dia->autosaved = FALSE;
+  if (diagram_is_modified(dia)) {
+    dia->autosaved = FALSE;
+    dia->is_default = FALSE;
+  }
   /*  diagram_set_modified(dia, TRUE);*/
 }
 
@@ -371,7 +393,8 @@ diagram_selected_any_parents(Diagram *dia) {
   for (selected = dia->data->selected;
        selected != NULL; selected = selected->next) {
     DiaObject *obj = (DiaObject*)selected->data;
-    if (obj->can_parent && obj->children != NULL) return TRUE;
+    if (object_flags_set(obj, DIA_OBJECT_CAN_PARENT) && obj->children != NULL) 
+      return TRUE;
   }
   return FALSE;
 }
@@ -399,7 +422,7 @@ diagram_selected_can_parent(Diagram *dia) {
   for (selected = dia->data->selected;
        selected != NULL; selected = selected->next) {
     DiaObject *obj = (DiaObject*)selected->data;
-    if (obj->can_parent) {
+    if (object_flags_set(obj, DIA_OBJECT_CAN_PARENT)) {
       parents = g_list_prepend(parents, obj);
     }
   }
@@ -446,48 +469,42 @@ diagram_update_menu_sensitivity (Diagram *dia, UpdatableMenuItems *items)
 {
   gint selected_count = g_list_length (dia->data->selected);
   /* Edit menu */
-  gtk_widget_set_sensitive(GTK_WIDGET(items->copy), selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->cut), selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->paste),
-			   cnp_exist_stored_objects());
-  gtk_widget_set_sensitive(GTK_WIDGET(items->edit_delete), selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->edit_duplicate), selected_count > 0);
+  gtk_action_set_sensitive (items->copy, selected_count > 0);
+  gtk_action_set_sensitive (items->cut, selected_count > 0);
+  gtk_action_set_sensitive (items->paste, cnp_exist_stored_objects());
+  gtk_action_set_sensitive (items->edit_delete, selected_count > 0);
+  gtk_action_set_sensitive (items->edit_duplicate, selected_count > 0);
 
-  gtk_widget_set_sensitive(GTK_WIDGET(items->copy_text),
-			   active_focus() != NULL);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->cut_text),
-			   active_focus() != NULL);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->paste_text),
-			   active_focus() != NULL);
+  gtk_action_set_sensitive (items->copy_text, active_focus() != NULL);
+  gtk_action_set_sensitive (items->cut_text, active_focus() != NULL);
+  gtk_action_set_sensitive (items->paste_text, active_focus() != NULL);
   
   /* Objects menu */
-  gtk_widget_set_sensitive(GTK_WIDGET(items->send_to_back), selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->bring_to_front), selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->send_backwards), selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->bring_forwards), selected_count > 0);
+  gtk_action_set_sensitive (items->send_to_back, selected_count > 0);
+  gtk_action_set_sensitive (items->bring_to_front, selected_count > 0);
+  gtk_action_set_sensitive (items->send_backwards, selected_count > 0);
+  gtk_action_set_sensitive (items->bring_forwards, selected_count > 0);
     
-  gtk_widget_set_sensitive(GTK_WIDGET(items->parent),
-			   diagram_selected_can_parent(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->unparent),
-			   diagram_selected_any_children(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->unparent_children),
-			   diagram_selected_any_parents(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->group), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->ungroup),
-			   diagram_selected_any_groups(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->properties), selected_count > 0);
+  gtk_action_set_sensitive (items->parent, diagram_selected_can_parent (dia));
+  gtk_action_set_sensitive (items->unparent, 
+			    diagram_selected_any_children (dia));
+  gtk_action_set_sensitive (items->unparent_children,
+			    diagram_selected_any_parents (dia));
+  gtk_action_set_sensitive (items->group, selected_count > 1);
+  gtk_action_set_sensitive (items->ungroup, diagram_selected_any_groups (dia));
+  gtk_action_set_sensitive (items->properties, selected_count > 0);
   
   /* Objects->Align menu */
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_l), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_c), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_r), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_e), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_a), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_t), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_c), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_b), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_e), selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_a), selected_count > 1);
+  gtk_action_set_sensitive (items->align_h_l, selected_count > 1);
+  gtk_action_set_sensitive (items->align_h_c, selected_count > 1);
+  gtk_action_set_sensitive (items->align_h_r, selected_count > 1);
+  gtk_action_set_sensitive (items->align_h_e, selected_count > 1);
+  gtk_action_set_sensitive (items->align_h_a, selected_count > 1);
+  gtk_action_set_sensitive (items->align_v_t, selected_count > 1);
+  gtk_action_set_sensitive (items->align_v_c, selected_count > 1);
+  gtk_action_set_sensitive (items->align_v_b, selected_count > 1);
+  gtk_action_set_sensitive (items->align_v_e, selected_count > 1);
+  gtk_action_set_sensitive (items->align_v_a, selected_count > 1);
 }
     
   
@@ -501,16 +518,12 @@ void diagram_update_popupmenu_sensitivity(Diagram *dia)
 {
   static int initialized = 0;
   static UpdatableMenuItems items;
-  
-
-  char *display = "<Display>";
-  
+ 
   if (initialized==0) {
-      menus_initialize_updatable_items (&items, NULL, display);
-      
+      menus_initialize_updatable_items (&items, NULL);      
       initialized = 1;
   }
-  
+
   diagram_update_menu_sensitivity (dia, &items);
 }
 
@@ -634,8 +647,9 @@ diagram_unselect_object(Diagram *diagram, DiaObject *obj)
 {
   object_add_updates(obj, diagram);
   textedit_remove_focus(obj, diagram);
-  data_unselect(diagram->data, obj);
-  g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0, g_list_length (diagram->data->selected));
+  data_unselect(DIA_DIAGRAM_DATA(diagram), obj);
+  g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0,
+		 g_list_length (DIA_DIAGRAM_DATA(diagram)->selected));
 }
 
 void
@@ -660,13 +674,22 @@ diagram_unselect_objects(Diagram *dia, GList *obj_list)
   g_signal_emit (dia, diagram_signals[SELECTION_CHANGED], 0, g_list_length (dia->data->selected));
 }
 
+/** Make a single object selected.
+ * Note that an object inside a closed group cannot be made selected, nor
+ * can an object in a non-active layer.
+ * @param diagram The diagram that the object belongs to (sorta redundant now)
+ * @param obj The object that should be made selected.
+ */
 void
 diagram_select(Diagram *diagram, DiaObject *obj)
 {
-  data_select(diagram->data, obj);
-  obj->ops->selectf(obj, NULL, NULL);
-  object_add_updates(obj, diagram);
-  g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0, g_list_length (diagram->data->selected));
+  if (dia_object_is_selectable(obj)) {
+    data_select(diagram->data, obj);
+    obj->ops->selectf(obj, NULL, NULL);
+    object_add_updates(obj, diagram);
+    g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0,
+		   g_list_length (diagram->data->selected));
+  }
 }
 
 void
@@ -976,7 +999,7 @@ void diagram_parent_selected(Diagram *dia)
     for (idx2 = idx + 1; idx2 < length; idx2++)
     {
       ObjectExtent *oe2 = g_ptr_array_index(extents, idx2);
-      if (!oe2->object->can_parent)
+      if (!object_flags_set(oe2->object, DIA_OBJECT_CAN_PARENT))
         continue;
 
       if (oe1->extent.right <= oe2->extent.right
@@ -1042,7 +1065,7 @@ void diagram_unparent_children_selected(Diagram *dia)
   for (list = dia->data->selected; list != NULL; list = g_list_next(list))
   {
     obj = (DiaObject *) list->data;
-    if (!obj->can_parent || !obj->children)
+    if (!object_flags_set(obj, DIA_OBJECT_CAN_PARENT) || !obj->children)
       continue;
 
     any_unparented = TRUE;
@@ -1077,6 +1100,11 @@ void diagram_group_selected(Diagram *dia)
   GList *orig_list;
   Change *change;
 
+  if (g_list_length(dia->data->selected) < 1) {
+    message_error("Trying to group with no selected objects.");
+    return;
+  }
+  
 #if 0
   /* the following is wrong as it screws up the selected list, see bug #153525
      * I just don't get what was originally intented so please speak up if you know  --hb
@@ -1099,21 +1127,6 @@ void diagram_group_selected(Diagram *dia)
     /* The connections aren't reattached by ungroup. */
     strip_connections(obj, dia->data->selected, dia);
     
-    /* Have to hide any open properties dialog
-     * if it contains some object in cut_list */
-    /* Now handled by undo_apply
-    properties_hide_if_shown(dia, obj);
-    */
-
-    /* Remove focus if active */
-    /* Now handled by undo apply
-       textedit_remove_focus(obj);
-
-    */
-
-    /* Now handled by the undo apply
-    object_add_updates(obj, dia);
-    */
     list = g_list_next(list);
   }
 
@@ -1124,13 +1137,13 @@ void diagram_group_selected(Diagram *dia)
   group = group_create(group_list);
   change = undo_group_objects(dia, group_list, group, orig_list);
   (change->apply)(change, dia);
-
+  
   /* Select the created group */
   diagram_select(dia, group);
   
   diagram_modified(dia);
   diagram_flush(dia);
-
+  
   undo_set_transactionpoint(dia->undo);
 }
 
@@ -1138,7 +1151,6 @@ void diagram_ungroup_selected(Diagram *dia)
 {
   DiaObject *group;
   GList *group_list;
-/*   GList *list; */
   GList *selected, *selection_copy;
   int group_index;
   int any_groups = 0;
@@ -1161,29 +1173,11 @@ void diagram_ungroup_selected(Diagram *dia)
 
       group_list = group_objects(group);
       diagram_select_list(dia, group_list);
-      /* Now handled by undo apply */      
-      /*list = group_list;
-      while (list != NULL) {
-	DiaObject *obj = (DiaObject *)list->data;
-	object_add_updates(obj, dia);
-	list = g_list_next(list);
-      }
-      */
-
-      /* Now handled by undo apply
-      layer_replace_object_with_list(dia->data->active_layer,
-				     group, g_list_copy(group_list));
-      */
 
       group_index = layer_object_index(dia->data->active_layer, group);
     
       change = undo_ungroup_objects(dia, group_list, group, group_index);
       (change->apply)(change, dia);
-      /* Now handled by undo apply 
-      diagram_tree_add_objects(diagram_tree(), dia, group_list);
-      diagram_tree_remove_object(diagram_tree(), group);
-      properties_hide_if_shown(dia, group);
-      */
 
       any_groups = 1;
     }
@@ -1351,13 +1345,25 @@ diagram_place_down_selected(Diagram *dia)
 
 void
 diagram_set_filename(Diagram *dia, const char *filename)
+{
+  g_free(dia->filename);
+  dia->filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+  
+  diagram_update_for_filename(dia);
+}
+
+/** Update the various areas that require updating when changing filename
+ * This will ensure that all places that use the filename are updated:
+ * Window titles, layer dialog, recent files, diagram tree.
+ * @param dia The diagram whose filename has changed.
+ */
+static void
+diagram_update_for_filename(Diagram *dia)
 {
   GSList *l;
   DDisplay *ddisp;
   char *title;
-  
-  g_free(dia->filename);
-  dia->filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+  char *filename = dia->filename;
 
   title = diagram_get_name(dia);
 
@@ -1385,14 +1391,7 @@ diagram_set_filename(Diagram *dia, const char *filename)
 gchar *
 diagram_get_name(Diagram *dia)
 {
-  gchar *title = strrchr(dia->filename, G_DIR_SEPARATOR);
-  if (title==NULL) {
-    title = dia->filename;
-  } else {
-    title++;
-  }
-  
-  return g_strdup(title);
+  return g_path_get_basename(dia->filename);
 }
 
 int diagram_modified_exists(void)
@@ -1417,11 +1416,3 @@ void diagram_object_modified(Diagram *dia, DiaObject *object)
 {
   diagram_tree_update_object(diagram_tree(), dia, object);
 }
-
-static 
-void close_dialog(gpointer data, gpointer user_data)
-{
-  if (data != NULL) {
-    g_signal_emit_by_name(G_OBJECT(data), "delete_event");
-  }
-}
