@@ -33,12 +33,14 @@
 #include "message.h"
 #include "properties.h"
 
-#include "pixmaps/box.xpm"
+#include "tool-icons.h"
 
 #define DEFAULT_WIDTH 2.0
 #define DEFAULT_HEIGHT 1.0
 #define DEFAULT_BORDER 0.25
 
+#define NUM_CONNECTIONS 9
+
 typedef enum {
   FREE_ASPECT,
   FIXED_ASPECT,
@@ -50,7 +52,7 @@ typedef struct _Box Box;
 struct _Box {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[NUM_CONNECTIONS];
 
   real border_width;
   Color border_color;
@@ -78,19 +80,19 @@ static ObjectChange* box_move_handle(Box *box, Handle *handle,
 static ObjectChange* box_move(Box *box, Point *to);
 static void box_draw(Box *box, DiaRenderer *renderer);
 static void box_update_data(Box *box);
-static Object *box_create(Point *startpoint,
+static DiaObject *box_create(Point *startpoint,
 			  void *user_data,
 			  Handle **handle1,
 			  Handle **handle2);
 static void box_destroy(Box *box);
-static Object *box_copy(Box *box);
+static DiaObject *box_copy(Box *box);
 
 static PropDescription *box_describe_props(Box *box);
 static void box_get_props(Box *box, GPtrArray *props);
 static void box_set_props(Box *box, GPtrArray *props);
 
 static void box_save(Box *box, ObjectNode obj_node, const char *filename);
-static Object *box_load(ObjectNode obj_node, int version, const char *filename);
+static DiaObject *box_load(ObjectNode obj_node, int version, const char *filename);
 static DiaMenu *box_get_object_menu(Box *box, Point *clickedpoint);
 
 static ObjectTypeOps box_type_ops =
@@ -102,16 +104,16 @@ static ObjectTypeOps box_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType box_type =
+DiaObjectType box_type =
 {
   "Standard - Box",  /* name */
   0,                 /* version */
-  (char **) box_xpm, /* pixmap */
+  (char **) box_icon, /* pixmap */
 
   &box_type_ops      /* ops */
 };
 
-ObjectType *_box_type = (ObjectType *) &box_type;
+DiaObjectType *_box_type = (DiaObjectType *) &box_type;
 
 static ObjectOps box_ops = {
   (DestroyFunc)         box_destroy,
@@ -144,7 +146,7 @@ static PropDescription box_props[] = {
   PROP_STD_FILL_COLOUR,
   PROP_STD_SHOW_BACKGROUND,
   PROP_STD_LINE_STYLE,
-  { "corner_radius", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+  { "corner_radius", PROP_TYPE_LENGTH, PROP_FLAG_VISIBLE,
     N_("Corner radius"), NULL, &corner_radius_data },
   { "aspect", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
     N_("Aspect ratio"), NULL, prop_aspect_data },
@@ -168,7 +170,7 @@ static PropOffset box_offsets[] = {
   { "aspect", PROP_TYPE_ENUM, offsetof(Box, aspect) },
   { "line_style", PROP_TYPE_LINESTYLE,
     offsetof(Box, line_style), offsetof(Box, dashlength) },
-  { "corner_radius", PROP_TYPE_REAL, offsetof(Box, corner_radius) },
+  { "corner_radius", PROP_TYPE_LENGTH, offsetof(Box, corner_radius) },
   { NULL, 0, 0 }
 };
 
@@ -240,7 +242,7 @@ box_move_handle(Box *box, Handle *handle,
     double new_width, new_height;
     double to_width, aspect_width;
     Point corner = box->element.corner;
-    Point nw_to, se_to;
+    Point se_to;
 
     width = box->element.width;
     height = box->element.height;
@@ -309,6 +311,11 @@ box_draw(Box *box, DiaRenderer *renderer)
   lr_corner.x = elem->corner.x + elem->width;
   lr_corner.y = elem->corner.y + elem->height;
 
+  renderer_ops->set_linewidth(renderer, box->border_width);
+  renderer_ops->set_linestyle(renderer, box->line_style);
+  renderer_ops->set_dashlength(renderer, box->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+
   if (box->show_background) {
     renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   
@@ -327,11 +334,6 @@ box_draw(Box *box, DiaRenderer *renderer)
     }
   }
 
-  renderer_ops->set_linewidth(renderer, box->border_width);
-  renderer_ops->set_linestyle(renderer, box->line_style);
-  renderer_ops->set_dashlength(renderer, box->dashlength);
-  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
-
   if (box->corner_radius > 0) {
     renderer_ops->draw_rounded_rect(renderer, 
 			     &elem->corner,
@@ -352,7 +354,7 @@ box_update_data(Box *box)
 {
   Element *elem = &box->element;
   ElementBBExtras *extra = &elem->extra_spacing;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   real radius;
   
   if (box->aspect == SQUARE_ASPECT){
@@ -382,6 +384,8 @@ box_update_data(Box *box)
   box->connections[6].pos.y = elem->corner.y + elem->height;
   box->connections[7].pos.x = elem->corner.x + elem->width - radius;
   box->connections[7].pos.y = elem->corner.y + elem->height - radius;
+  box->connections[8].pos.x = elem->corner.x + elem->width / 2.0;
+  box->connections[8].pos.y = elem->corner.y + elem->height / 2.0;
 
   box->connections[0].directions = DIR_NORTH|DIR_WEST;
   box->connections[1].directions = DIR_NORTH;
@@ -391,6 +395,7 @@ box_update_data(Box *box)
   box->connections[5].directions = DIR_SOUTH|DIR_WEST;
   box->connections[6].directions = DIR_SOUTH;
   box->connections[7].directions = DIR_SOUTH|DIR_EAST;
+  box->connections[8].directions = DIR_ALL;
 
   extra->border_trans = box->border_width / 2.0;
   element_update_boundingbox(elem);
@@ -412,7 +417,7 @@ box_update_data(Box *box)
   }
 }
 
-static Object *
+static DiaObject *
 box_create(Point *startpoint,
 	   void *user_data,
 	   Handle **handle1,
@@ -420,7 +425,7 @@ box_create(Point *startpoint,
 {
   Box *box;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   int i;
 
   box = g_malloc0(sizeof(Box));
@@ -431,7 +436,6 @@ box_create(Point *startpoint,
 
   obj->ops = &box_ops;
 
-
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
   elem->height = DEFAULT_HEIGHT;
@@ -445,13 +449,14 @@ box_create(Point *startpoint,
   box->corner_radius = default_properties.corner_radius;
   box->aspect = default_properties.aspect;
 
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
 
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &box->connections[i];
     box->connections[i].object = obj;
     box->connections[i].connected = NULL;
   }
+  box->connections[8].flags = CP_FLAGS_MAIN;
 
   box_update_data(box);
 
@@ -466,13 +471,13 @@ box_destroy(Box *box)
   element_destroy(&box->element);
 }
 
-static Object *
+static DiaObject *
 box_copy(Box *box)
 {
   int i;
   Box *newbox;
   Element *elem, *newelem;
-  Object *newobj;
+  DiaObject *newobj;
   
   elem = &box->element;
   
@@ -491,12 +496,13 @@ box_copy(Box *box)
   newbox->corner_radius = box->corner_radius;
   newbox->aspect = box->aspect;
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     newobj->connections[i] = &newbox->connections[i];
     newbox->connections[i].object = newobj;
     newbox->connections[i].connected = NULL;
     newbox->connections[i].pos = box->connections[i].pos;
     newbox->connections[i].last_pos = box->connections[i].last_pos;
+    newbox->connections[i].flags = box->connections[i].flags;
   }
 
   return &newbox->element.object;
@@ -539,12 +545,12 @@ box_save(Box *box, ObjectNode obj_node, const char *filename)
 		  box->aspect);
 }
 
-static Object *
+static DiaObject *
 box_load(ObjectNode obj_node, int version, const char *filename)
 {
   Box *box;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   int i;
   AttributeNode attr;
 
@@ -597,13 +603,14 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     box->aspect = data_enum(attribute_first_data(attr));
 
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
 
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &box->connections[i];
     box->connections[i].object = obj;
     box->connections[i].connected = NULL;
   }
+  box->connections[8].flags = CP_FLAGS_MAIN;
 
   box_update_data(box);
 
@@ -627,7 +634,7 @@ aspect_change_free(struct AspectChange *change)
 }
 
 static void
-aspect_change_apply(struct AspectChange *change, Object *obj)
+aspect_change_apply(struct AspectChange *change, DiaObject *obj)
 {
   Box *box = (Box*)obj;
 
@@ -636,7 +643,7 @@ aspect_change_apply(struct AspectChange *change, Object *obj)
 }
 
 static void
-aspect_change_revert(struct AspectChange *change, Object *obj)
+aspect_change_revert(struct AspectChange *change, DiaObject *obj)
 {
   Box *box = (Box*)obj;
 
@@ -652,7 +659,7 @@ aspect_create_change(Box *box, AspectType aspect)
 {
   struct AspectChange *change;
 
-  change = g_new(struct AspectChange, 1);
+  change = g_new0(struct AspectChange, 1);
 
   change->obj_change.apply = (ObjectChangeApplyFunc) aspect_change_apply;
   change->obj_change.revert = (ObjectChangeRevertFunc) aspect_change_revert;
@@ -669,7 +676,7 @@ aspect_create_change(Box *box, AspectType aspect)
 
 
 static ObjectChange *
-box_set_aspect_callback (Object* obj, Point* clicked, gpointer data)
+box_set_aspect_callback (DiaObject* obj, Point* clicked, gpointer data)
 {
   ObjectChange *change;
 
