@@ -35,7 +35,7 @@
 #include "connpoint_line.h"
 #include "properties.h"
 
-#include "pixmaps/line.xpm"
+#include "tool-icons.h"
 
 #define DEFAULT_WIDTH 0.25
 
@@ -52,14 +52,10 @@ typedef struct _Line {
   Arrow start_arrow, end_arrow;
   real dashlength;
   real absolute_start_gap, absolute_end_gap;
-  real fractional_start_gap, fractional_end_gap;
-  gboolean object_edge_start, object_edge_end;
 } Line;
 
 struct _LineProperties {
   real absolute_start_gap, absolute_end_gap;
-  real fractional_start_gap, fractional_end_gap;
-  gboolean object_edge_start, object_edge_end;
 };
 
 static LineProperties default_properties;
@@ -72,24 +68,24 @@ static ObjectChange* line_move(Line *line, Point *to);
 static void line_select(Line *line, Point *clicked_point,
 			DiaRenderer *interactive_renderer);
 static void line_draw(Line *line, DiaRenderer *renderer);
-static Object *line_create(Point *startpoint,
+static DiaObject *line_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
 			   Handle **handle2);
 static real line_distance_from(Line *line, Point *point);
 static void line_update_data(Line *line);
 static void line_destroy(Line *line);
-static Object *line_copy(Line *line);
+static DiaObject *line_copy(Line *line);
 
 static PropDescription *line_describe_props(Line *line);
 static void line_get_props(Line *line, GPtrArray *props);
 static void line_set_props(Line *line, GPtrArray *props);
 
 static void line_save(Line *line, ObjectNode obj_node, const char *filename);
-static Object *line_load(ObjectNode obj_node, int version, const char *filename);
+static DiaObject *line_load(ObjectNode obj_node, int version, const char *filename);
 static DiaMenu *line_get_object_menu(Line *line, Point *clickedpoint);
 
-void calculate_gap_endpoints(Line *line, Point *gap_endpoints);
+void Line_adjust_for_absolute_gap(Line *line, Point *gap_endpoints);
 
 static ObjectTypeOps line_type_ops =
 {
@@ -100,15 +96,15 @@ static ObjectTypeOps line_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType line_type =
+DiaObjectType line_type =
 {
   "Standard - Line",   /* name */
   0,                   /* version */
-  (char **) line_xpm,  /* pixmap */
+  (char **) line_icon,  /* pixmap */
   &line_type_ops       /* ops */
 };
 
-ObjectType *_line_type = (ObjectType *) &line_type;
+DiaObjectType *_line_type = (DiaObjectType *) &line_type;
 
 static ObjectOps line_ops = {
   (DestroyFunc)         line_destroy,
@@ -141,22 +137,14 @@ static PropDescription line_props[] = {
     N_("Start point"), NULL },
   { "end_point", PROP_TYPE_POINT, 0,
     N_("End point"), NULL },
-  /*
+  
   PROP_FRAME_BEGIN("gaps",0,N_("Line gaps")),
-  { "absolute_start_gap", PROP_TYPE_REAL, 0,
+  { "absolute_start_gap", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
     N_("Absolute start gap"), NULL, &gap_range },
-  { "absolute_end_gap", PROP_TYPE_REAL, 0,
+  { "absolute_end_gap", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
     N_("Absolute end gap"), NULL, &gap_range },
-  { "fractional_start_gap", PROP_TYPE_REAL, 0,
-    N_("Fractional start gap"), NULL, &gap_range },
-  { "fractional_end_gap", PROP_TYPE_REAL, 0,
-    N_("Fractional end gap"), NULL, &gap_range },
-  { "object_edge_start", PROP_TYPE_BOOL, 0,
-    N_("Start at object edge"), },
-  { "object_edge_end", PROP_TYPE_BOOL, 0,
-    N_("End at object edge"), },
   PROP_FRAME_END("gaps",0),
-  */
+ 
   PROP_DESC_END
 };
 
@@ -178,14 +166,8 @@ static PropOffset line_offsets[] = {
   { "end_arrow", PROP_TYPE_ARROW, offsetof(Line, end_arrow) },
   { "start_point", PROP_TYPE_POINT, offsetof(Connection, endpoints[0]) },
   { "end_point", PROP_TYPE_POINT, offsetof(Connection, endpoints[1]) },
-  /*
   { "absolute_start_gap", PROP_TYPE_REAL, offsetof(Line, absolute_start_gap) },
   { "absolute_end_gap", PROP_TYPE_REAL, offsetof(Line, absolute_end_gap) },
-  { "fractional_start_gap", PROP_TYPE_REAL, offsetof(Line, fractional_start_gap) },
-  { "fractional_end_gap", PROP_TYPE_REAL, offsetof(Line, fractional_end_gap) },
-  { "object_edge_start", PROP_TYPE_BOOL, offsetof(Line, object_edge_start) },
-  { "object_edge_end", PROP_TYPE_BOOL, offsetof(Line, object_edge_end) },
-  */
   { NULL, 0, 0 }
 };
 
@@ -211,16 +193,12 @@ line_init_defaults() {
   if (!defaults_initialized) {
     default_properties.absolute_start_gap = 0.0;
     default_properties.absolute_end_gap = 0.0;
-    default_properties.fractional_start_gap = 0;
-    default_properties.fractional_end_gap = 0;
-    default_properties.object_edge_start = FALSE;
-    default_properties.object_edge_end = FALSE;
     defaults_initialized = 1;
   }
 }
 
 static ObjectChange *
-line_add_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
+line_add_connpoint_callback(DiaObject *obj, Point *clicked, gpointer data) 
 {
   ObjectChange *oc;
   oc = connpointline_add_point(((Line *)obj)->cpl,clicked);
@@ -229,7 +207,7 @@ line_add_connpoint_callback(Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-line_remove_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
+line_remove_connpoint_callback(DiaObject *obj, Point *clicked, gpointer data) 
 {
   ObjectChange *oc;
   oc = connpointline_remove_point(((Line *)obj)->cpl,clicked);
@@ -265,22 +243,18 @@ line_get_object_menu(Line *line, Point *clickedpoint)
 }
 
 /*
-  About start_gap, end_gap, start_fractional_gap, end_fractional_gap
+  About start_gap, end_gap, auto/absolute
   
-  Place positive reals (try 1.0, for instance) to create gaps on line
-  end/start.  If fractional_gap is false, these gaps are of fixed
-  length.  If fractional_gap is true, these gaps are a fraction of
-  total line length.  I don't know if fractional gaps are really
-  useful, but since it doesn't complicate the code, why not?
-  If a gap are negative, the line extends past the handle.
+  Place positive reals (try 1.0, for instance) in absolute 
+  to create gaps on line end/start.  
+  If auto_gap is false, these gaps are of that length.
+  If a gap is negative, the line extends past the handle.
+  If auto_gap is true, the gap length is computed so it touches 
+  the connected object edge then the absolute gap is added
+  */
   
-  Gaps as implemented for now do not use any object intersection
-  code.  It would be nice to have gap code that combined a fixed gap
-  length which could be optionally added to the object intersection
-  gap.
-*/
 Point
-calculate_object_edge(Point *objmid, Point *end, Object *obj) 
+calculate_object_edge(Point *objmid, Point *end, DiaObject *obj) 
 {
 #define MAXITER 25
 #ifdef TRACE_DIST
@@ -329,8 +303,13 @@ calculate_object_edge(Point *objmid, Point *end, Object *obj)
   return mid2;
 }
 
+
+/** Calculate the absolute gap -- this gap is 'transient', in that
+ * the actual end of the line is not moved, but it is made to look like
+ * it is shorter.
+ */
 void
-calculate_gap_endpoints(Line *line, Point *gap_endpoints)
+line_adjust_for_absolute_gap(Line *line, Point *gap_endpoints)
 {
   Point endpoints[2];
   real line_length;
@@ -338,30 +317,15 @@ calculate_gap_endpoints(Line *line, Point *gap_endpoints)
   endpoints[0] = line->connection.endpoints[0];
   endpoints[1] = line->connection.endpoints[1];
 
-  if (line->object_edge_start &&
-      line->connection.endpoint_handles[0].connected_to != NULL) {
-      endpoints[0] = calculate_object_edge(&line->connection.endpoints[0],
-					   &line->connection.endpoints[1],
-					   line->connection.endpoint_handles[0].connected_to->object);
-  }
-  if (line->object_edge_end &&
-      line->connection.endpoint_handles[1].connected_to != NULL) {
-      endpoints[1] = calculate_object_edge(&line->connection.endpoints[1],
-					   &line->connection.endpoints[0],
-					   line->connection.endpoint_handles[1].connected_to->object);
-  }
+  line_length = distance_point_point(&endpoints[0], &endpoints[1]);
 
-  line_length = distance_point_point(&endpoints[0],&endpoints[1]);
+  /* puts new 0 to x% of  0->1  */
+  point_convex(&gap_endpoints[0], &endpoints[0], &endpoints[1],
+              1 - line->absolute_start_gap/line_length);
 
-  point_convex(&gap_endpoints[0],&endpoints[1],&endpoints[0],
-              line->absolute_start_gap/line_length +
-              line->fractional_start_gap
-              );
-
-  point_convex(&gap_endpoints[1],&endpoints[0],&endpoints[1],
-              line->absolute_end_gap/line_length +
-              line->fractional_end_gap
-              );
+  /* puts new 1 to x% of  1->0  */
+  point_convex(&gap_endpoints[1], &endpoints[1], &endpoints[0],
+              1 - line->absolute_end_gap/line_length);
 }
 
 static real
@@ -371,12 +335,10 @@ line_distance_from(Line *line, Point *point)
 
   endpoints = &line->connection.endpoints[0]; 
 
-  if (line->object_edge_start || line->object_edge_end ||
-      line->absolute_start_gap || line->absolute_end_gap ||
-      line->fractional_start_gap || line->fractional_end_gap) {
+  if (line->absolute_start_gap || line->absolute_end_gap ) {
     Point gap_endpoints[2];  /* Visible endpoints of line */
 
-    calculate_gap_endpoints(line, gap_endpoints);
+    line_adjust_for_absolute_gap(line, gap_endpoints);
     return distance_line_point( &gap_endpoints[0], &gap_endpoints[1],
                                line->line_width, point);
   } else {
@@ -440,10 +402,8 @@ line_draw(Line *line, DiaRenderer *renderer)
   renderer_ops->set_dashlength(renderer, line->dashlength);
   renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  if (line->object_edge_start || line->object_edge_end ||
-      line->absolute_start_gap || line->absolute_end_gap ||
-      line->fractional_start_gap || line->fractional_end_gap) {
-    calculate_gap_endpoints(line, gap_endpoints);
+  if (line->absolute_start_gap || line->absolute_end_gap ) {
+    line_adjust_for_absolute_gap(line, gap_endpoints);
 
     renderer_ops->draw_line_with_arrows(renderer,
 					&gap_endpoints[0], &gap_endpoints[1],
@@ -463,7 +423,7 @@ line_draw(Line *line, DiaRenderer *renderer)
   }
 }
 
-static Object *
+static DiaObject *
 line_create(Point *startpoint,
 	    void *user_data,
 	    Handle **handle1,
@@ -471,7 +431,7 @@ line_create(Point *startpoint,
 {
   Line *line;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
   line_init_defaults();
@@ -482,10 +442,6 @@ line_create(Point *startpoint,
   line->line_color = attributes_get_foreground();
   line->absolute_start_gap = default_properties.absolute_start_gap;
   line->absolute_end_gap = default_properties.absolute_end_gap;
-  line->fractional_start_gap = default_properties.fractional_start_gap;
-  line->fractional_end_gap = default_properties.fractional_end_gap;
-  line->object_edge_start = default_properties.object_edge_start;
-  line->object_edge_end = default_properties.object_edge_end;
     
   conn = &line->connection;
   conn->endpoints[0] = *startpoint;
@@ -517,12 +473,12 @@ line_destroy(Line *line)
   connection_destroy(&line->connection);
 }
 
-static Object *
+static DiaObject *
 line_copy(Line *line)
 {
   Line *newline;
   Connection *conn, *newconn;
-  Object *newobj;
+  DiaObject *newobj;
   int rcc = 0;
 
   conn = &line->connection;
@@ -543,10 +499,6 @@ line_copy(Line *line)
   newline->end_arrow = line->end_arrow;
   newline->absolute_start_gap = line->absolute_start_gap;
   newline->absolute_end_gap = line->absolute_end_gap;
-  newline->fractional_start_gap = line->fractional_start_gap;
-  newline->fractional_end_gap = line->fractional_end_gap;
-  newline->object_edge_start = line->object_edge_start;
-  newline->object_edge_end = line->object_edge_end;
 
   line_update_data(line);
 
@@ -557,7 +509,7 @@ static void
 line_update_data(Line *line)
 {
   Connection *conn = &line->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   LineBBExtras *extra = &conn->extra_spacing;
   Point start, end;
 
@@ -570,12 +522,14 @@ line_update_data(Line *line)
   if (line->end_arrow.type != ARROW_NONE) 
     extra->end_trans = MAX(extra->end_trans,line->end_arrow.width);
 
-  if (line->object_edge_start || line->object_edge_end ||
-      line->absolute_start_gap || line->absolute_end_gap ||
-      line->fractional_start_gap || line->fractional_end_gap) {
+  if (connpoint_is_autogap(line->connection.endpoint_handles[0].connected_to) ||
+      connpoint_is_autogap(line->connection.endpoint_handles[1].connected_to)) {
+    connection_adjust_for_autogap(line);
+  }
+  if (line->absolute_start_gap || line->absolute_end_gap ) {
     Point gap_endpoints[2];
 
-    calculate_gap_endpoints(line, gap_endpoints);
+    line_adjust_for_absolute_gap(line, gap_endpoints);
     line_bbox(&gap_endpoints[0],&gap_endpoints[1],
 	      &conn->extra_spacing,&conn->object.bounding_box);
     start = gap_endpoints[0];
@@ -598,6 +552,10 @@ line_update_data(Line *line)
 static void
 line_save(Line *line, ObjectNode obj_node, const char *filename)
 {
+#ifdef DEBUG
+  dia_object_sanity_check((DiaObject*)line, "Saving line");
+#endif
+
   connection_save(&line->connection, obj_node);
 
   connpointline_save(line->cpl,obj_node,"numcp");
@@ -615,21 +573,13 @@ line_save(Line *line, ObjectNode obj_node, const char *filename)
 		  line->line_style);
   
   if (line->start_arrow.type != ARROW_NONE) {
-    data_add_enum(new_attribute(obj_node, "start_arrow"),
-		  line->start_arrow.type);
-    data_add_real(new_attribute(obj_node, "start_arrow_length"),
-		  line->start_arrow.length);
-    data_add_real(new_attribute(obj_node, "start_arrow_width"),
-		  line->start_arrow.width);
+    save_arrow(obj_node, &line->start_arrow,
+	       "start_arrow", "start_arrow_length", "start_arrow_width");
   }
   
   if (line->end_arrow.type != ARROW_NONE) {
-    data_add_enum(new_attribute(obj_node, "end_arrow"),
-		  line->end_arrow.type);
-    data_add_real(new_attribute(obj_node, "end_arrow_length"),
-		  line->end_arrow.length);
-    data_add_real(new_attribute(obj_node, "end_arrow_width"),
-		  line->end_arrow.width);
+    save_arrow(obj_node, &line->end_arrow,
+	       "end_arrow", "end_arrow_length", "end_arrow_width");
   }
  
   if (line->absolute_start_gap)
@@ -638,30 +588,18 @@ line_save(Line *line, ObjectNode obj_node, const char *filename)
   if (line->absolute_end_gap)
     data_add_real(new_attribute(obj_node, "absolute_end_gap"),
                  line->absolute_end_gap);
-  if (line->fractional_start_gap)
-    data_add_real(new_attribute(obj_node, "fractional_start_gap"),
-                 line->fractional_start_gap);
-  if (line->fractional_end_gap)
-    data_add_real(new_attribute(obj_node, "fractional_end_gap"),
-                 line->fractional_end_gap);
-  if (line->object_edge_start)
-    data_add_boolean(new_attribute(obj_node, "object_edge_start"),
-		     line->object_edge_start);
-  if (line->object_edge_end)
-    data_add_boolean(new_attribute(obj_node, "object_edge_end"),
-		     line->object_edge_end);
 
   if (line->line_style != LINESTYLE_SOLID && line->dashlength != DEFAULT_LINESTYLE_DASHLEN)
     data_add_real(new_attribute(obj_node, "dashlength"),
 		  line->dashlength);
 }
 
-static Object *
+static DiaObject *
 line_load(ObjectNode obj_node, int version, const char *filename)
 {
   Line *line;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
 
   line = g_malloc0(sizeof(Line));
@@ -689,31 +627,11 @@ line_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     line->line_style = data_enum(attribute_first_data(attr));
 
-  line->start_arrow.type = ARROW_NONE;
-  line->start_arrow.length = DEFAULT_ARROW_LENGTH;
-  line->start_arrow.width = DEFAULT_ARROW_WIDTH;
-  attr = object_find_attribute(obj_node, "start_arrow");
-  if (attr != NULL)
-    line->start_arrow.type = data_enum(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "start_arrow_length");
-  if (attr != NULL)
-    line->start_arrow.length = data_real(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "start_arrow_width");
-  if (attr != NULL)
-    line->start_arrow.width = data_real(attribute_first_data(attr));
+  load_arrow(obj_node, &line->start_arrow, 
+	     "start_arrow", "start_arrow_length", "start_arrow_width");
 
-  line->end_arrow.type = ARROW_NONE;
-  line->end_arrow.length = DEFAULT_ARROW_LENGTH;
-  line->end_arrow.width = DEFAULT_ARROW_WIDTH;
-  attr = object_find_attribute(obj_node, "end_arrow");
-  if (attr != NULL)
-    line->end_arrow.type = data_enum(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "end_arrow_length");
-  if (attr != NULL)
-    line->end_arrow.length = data_real(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "end_arrow_width");
-  if (attr != NULL)
-    line->end_arrow.width = data_real(attribute_first_data(attr));
+  load_arrow(obj_node, &line->end_arrow, 
+	     "end_arrow", "end_arrow_length", "end_arrow_width");
 
   line->absolute_start_gap = 0.0;
   attr = object_find_attribute(obj_node, "absolute_start_gap");
@@ -723,22 +641,6 @@ line_load(ObjectNode obj_node, int version, const char *filename)
   attr = object_find_attribute(obj_node, "absolute_end_gap");
   if (attr != NULL)
     line->absolute_end_gap =  data_real( attribute_first_data(attr) );
-  line->fractional_start_gap = 0;
-  attr = object_find_attribute(obj_node, "fractional_start_gap");
-  if (attr != NULL)
-    line->fractional_start_gap =  data_real( attribute_first_data(attr) );
-  line->fractional_end_gap = 0;
-  attr = object_find_attribute(obj_node, "fractional_end_gap");
-  if (attr != NULL)
-    line->fractional_end_gap =  data_real( attribute_first_data(attr) );
-  line->object_edge_start = FALSE;
-  attr = object_find_attribute(obj_node, "object_edge_start");
-  if (attr != NULL)
-    line->object_edge_start = data_boolean( attribute_first_data(attr) );
-  line->object_edge_end = FALSE;
-  attr = object_find_attribute(obj_node, "object_edge_end");
-  if (attr != NULL)
-    line->object_edge_end = data_boolean( attribute_first_data(attr) );
 
   line->dashlength = DEFAULT_LINESTYLE_DASHLEN;
   attr = object_find_attribute(obj_node, "dashlength");
